@@ -16,13 +16,12 @@ class ParseAtom
   parser Parser::Atom
 end
 
-
 class OnlyParseAtom
   include HTTParty
 
   # Only support Atom
   class Parser::OnlyAtom < HTTParty::Parser
-    SupportedFormats = {"application/atom+xml" => :atom}
+    SupportedFormats = { "application/atom+xml" => :atom }
 
     protected
 
@@ -35,7 +34,6 @@ class OnlyParseAtom
   parser Parser::OnlyAtom
 end
 
-
 class SkipParsing
   include HTTParty
 
@@ -49,11 +47,10 @@ class SkipParsing
   parser Parser::Simple
 end
 
-
 class AdHocParsing
   include HTTParty
   parser(
-    Proc.new do |body, format|
+    proc do |body, format|
       case format
       when :json
         body.to_json
