@@ -3,4 +3,4 @@
 require 'httparty'
 
 APPLICATION_NAME = "Httparty"
-response = HTTParty.get('http://example.com', :headers => {"User-Agent" => APPLICATION_NAME})
+response = HTTParty.get('http://example.com', headers: {"User-Agent" => APPLICATION_NAME})
