@@ -1,6 +1,5 @@
 class HTTParty::CookieHash < Hash #:nodoc:
-
-  CLIENT_COOKIES = %w{path expires domain path secure HTTPOnly}
+  CLIENT_COOKIES = %w(path expires domain path secure httponly)
 
   def add_cookies(value)
     case value
@@ -8,7 +7,7 @@ class HTTParty::CookieHash < Hash #:nodoc:
       merge!(value)
     when String
       value.split('; ').each do |cookie|
-        array = cookie.split('=',2)
+        array = cookie.split('=', 2)
         self[array[0].to_sym] = array[1]
       end
     else
@@ -17,6 +16,6 @@ class HTTParty::CookieHash < Hash #:nodoc:
   end
 
   def to_cookie_string
-    delete_if { |k, v| CLIENT_COOKIES.include?(k.to_s.downcase) }.collect { |k, v| "#{k}=#{v}" }.join("; ")
+    reject { |k, v| CLIENT_COOKIES.include?(k.to_s.downcase) }.collect { |k, v| "#{k}=#{v}" }.join("; ")
   end
 end
