@@ -1,3 +1,5 @@
+require 'erb'
+
 module HTTParty
   class Request #:nodoc:
     SupportedHTTPMethods = [
@@ -9,7 +11,8 @@ module HTTParty
       Net::HTTP::Head,
       Net::HTTP::Options,
       Net::HTTP::Move,
-      Net::HTTP::Copy
+      Net::HTTP::Copy,
+      Net::HTTP::Mkcol,
     ]
 
     SupportedURISchemes  = ['http', 'https', 'webcal', nil]
@@ -26,10 +29,26 @@ module HTTParty
       end.flatten.join('&')
     end
 
+    JSON_API_QUERY_STRING_NORMALIZER = proc do |query|
+      Array(query).sort_by { |a| a[0].to_s }.map do |key, value|
+        if value.nil?
+          key.to_s
+        elsif value.respond_to?(:to_ary)
+          values = value.to_ary.map{|v| ERB::Util.url_encode(v.to_s)}
+          "#{key}=#{values.join(',')}"
+        else
+          HashConversions.to_params(key => value)
+        end
+      end.flatten.join('&')
+    end
+
     attr_accessor :http_method, :options, :last_response, :redirect, :last_uri
     attr_reader :path
 
     def initialize(http_method, path, o = {})
+      @changed_hosts = false
+      @credentials_sent = false
+
       self.http_method = http_method
       self.options = {
         limit: o.delete(:no_follow) ? 1 : 5,
@@ -73,7 +92,13 @@ module HTTParty
         path.path = last_uri_host + path.path
       end
 
-      new_uri = path.relative? ? options[:uri_adapter].parse("#{base_uri}#{path}") : path.clone
+      if path.relative? && path.host
+        new_uri = options[:uri_adapter].parse("#{@last_uri.scheme}:#{path}")
+      elsif path.relative?
+        new_uri = options[:uri_adapter].parse("#{base_uri}#{path}")
+      else
+        new_uri = path.clone
+      end
 
       # avoid double query string on redirects [#12]
       unless redirect
@@ -93,7 +118,7 @@ module HTTParty
         base_uri += ":#{@last_uri.port}" if @last_uri.port != 80
         base_uri
       else
-        options[:base_uri]
+        options[:base_uri] && HTTParty.normalize_base_uri(options[:base_uri])
       end
     end
 
@@ -127,8 +152,18 @@ module HTTParty
         end
       end
 
-      handle_deflation unless http_method == Net::HTTP::Head
-      handle_response(chunked_body, &block)
+
+      handle_host_redirection if response_redirects?
+      result = handle_unauthorized
+      result ||= handle_response(chunked_body, &block)
+      result
+    end
+
+    def handle_unauthorized(&block)
+      return unless digest_auth? && response_unauthorized? && response_has_digest_auth_challenge?
+      return if @credentials_sent
+      @credentials_sent = true
+      perform(&block)
     end
 
     def raw_body
@@ -173,19 +208,39 @@ module HTTParty
       @raw_request = http_method.new(request_uri(uri))
       @raw_request.body = body if body
       @raw_request.body_stream = options[:body_stream] if options[:body_stream]
-      @raw_request.initialize_http_header(options[:headers].to_hash) if options[:headers].respond_to?(:to_hash)
-      @raw_request.basic_auth(username, password) if options[:basic_auth]
-      setup_digest_auth if options[:digest_auth]
+      if options[:headers].respond_to?(:to_hash)
+        headers_hash = options[:headers].to_hash
+
+        @raw_request.initialize_http_header(headers_hash)
+        # If the caller specified a header of 'Accept-Encoding', assume they want to
+        # deal with encoding of content. Disable the internal logic in Net:HTTP
+        # that handles encoding, if the platform supports it.
+        if @raw_request.respond_to?(:decode_content) && (headers_hash.key?('Accept-Encoding') || headers_hash.key?('accept-encoding'))
+          # Using the '[]=' sets decode_content to false
+          @raw_request['accept-encoding'] = @raw_request['accept-encoding']
+        end
+      end
+      if options[:basic_auth] && send_authorization_header?
+        @raw_request.basic_auth(username, password)
+        @credentials_sent = true
+      end
+      setup_digest_auth if digest_auth? && response_unauthorized? && response_has_digest_auth_challenge?
+    end
+
+    def digest_auth?
+      !!options[:digest_auth]
+    end
+
+    def response_unauthorized?
+      !!last_response && last_response.code == '401'
+    end
+
+    def response_has_digest_auth_challenge?
+      !last_response['www-authenticate'].nil? && last_response['www-authenticate'].length > 0
     end
 
     def setup_digest_auth
-      auth_request = http_method.new(uri.request_uri)
-      auth_request.initialize_http_header(options[:headers].to_hash) if options[:headers].respond_to?(:to_hash)
-      res = http.request(auth_request)
-
-      if !res['www-authenticate'].nil? && res['www-authenticate'].length > 0
-        @raw_request.digest_auth(username, password, res)
-      end
+      @raw_request.digest_auth(username, password, last_response)
     end
 
     def query_string(uri)
@@ -222,8 +277,8 @@ module HTTParty
 
     def encode_with_ruby_encoding(body, charset)
       encoding = Encoding.find(charset)
-      body.force_encoding(encoding)
-    rescue
+      body.force_encoding(charset)
+    rescue ArgumentError
       body
     end
 
@@ -291,24 +346,29 @@ module HTTParty
         perform(&block)
       else
         body ||= last_response.body
-        body = encode_body(body)
+        body = body.nil? ? body : encode_body(body)
         Response.new(self, last_response, lambda { parse_response(body) }, body: body)
       end
     end
 
-    # Inspired by Ruby 1.9
-    def handle_deflation
-      case last_response["content-encoding"]
-      when "gzip", "x-gzip"
-        body_io = StringIO.new(last_response.body)
-        last_response.body.replace Zlib::GzipReader.new(body_io).read
-        last_response.delete('content-encoding')
-      when "deflate"
-        last_response.body.replace Zlib::Inflate.inflate(last_response.body)
-        last_response.delete('content-encoding')
+    def handle_host_redirection
+      check_duplicate_location_header
+      redirect_path = options[:uri_adapter].parse last_response['location']
+      return if redirect_path.relative? || path.host == redirect_path.host
+      @changed_hosts = true
+    end
+
+    def check_duplicate_location_header
+      location = last_response.get_fields('location')
+      if location.is_a?(Array) && location.count > 1
+        raise DuplicateLocationHeader.new(last_response)
       end
     end
 
+    def send_authorization_header?
+      !@changed_hosts
+    end
+
     def response_redirects?
       case last_response
       when Net::HTTPNotModified # 304
@@ -327,6 +387,7 @@ module HTTParty
       cookies_hash = HTTParty::CookieHash.new
       cookies_hash.add_cookies(options[:headers].to_hash['Cookie']) if options[:headers] && options[:headers].to_hash['Cookie']
       response.get_fields('Set-Cookie').each { |cookie| cookies_hash.add_cookies(cookie) }
+
       options[:headers] ||= {}
       options[:headers]['Cookie'] = cookies_hash.to_cookie_string
     end
@@ -358,6 +419,7 @@ module HTTParty
       if path.userinfo
         username, password = path.userinfo.split(':')
         options[:basic_auth] = {username: username, password: password}
+        @credentials_sent = true
       end
     end
   end
