@@ -1,5 +1,5 @@
 module HTTParty
-  class Response < BasicObject
+  class Response < Object
     def self.underscore(string)
       string.gsub(/([A-Z]+)([A-Z][a-z])/, '\1_\2').gsub(/([a-z])([A-Z])/, '\1_\2').downcase
     end
@@ -17,16 +17,14 @@ module HTTParty
         logger = ::HTTParty::Logger.build(request.options[:logger], request.options[:log_level], request.options[:log_format])
         logger.format(request, self)
       end
+
+      throw_exception
     end
 
     def parsed_response
       @parsed_response ||= @parsed_block.call
     end
 
-    def class
-      Response
-    end
-
     def code
       response.code.to_i
     end
@@ -45,7 +43,9 @@ module HTTParty
 
     CODES_TO_OBJ.each do |response_code, klass|
       name = klass.name.sub("Net::HTTP", '')
-      define_method("#{underscore(name)}?") do
+      name = "#{underscore(name)}?".to_sym
+
+      define_method(name) do
         klass === response
       end
     end
@@ -55,11 +55,33 @@ module HTTParty
       alias_method :multiple_choice?, :multiple_choices?
     end
 
-    def respond_to?(name, include_all = false)
-      return true if [:request, :response, :parsed_response, :body, :headers].include?(name)
-      parsed_response.respond_to?(name, include_all) || response.respond_to?(name, include_all)
+    def nil?
+      response.nil? || response.body.nil? || response.body.empty?
     end
 
+    def to_s      
+      if !response.nil? && !response.body.nil? && response.body.respond_to?(:to_s)
+        response.body.to_s
+      else 
+        inspect
+      end
+    end
+
+    def display(port=$>)
+      if !parsed_response.nil? && parsed_response.respond_to?(:display)
+        parsed_response.display(port)
+      elsif !response.nil? && !response.body.nil? && response.body.respond_to?(:display)
+        response.body.display(port)
+      else 
+        port.write(inspect)
+      end
+    end
+
+    def respond_to_missing?(name, *args)
+      return true if super
+      parsed_response.respond_to?(name) || response.respond_to?(name)
+    end
+    
     protected
 
     def method_missing(name, *args, &block)
@@ -71,6 +93,12 @@ module HTTParty
         super
       end
     end
+
+    def throw_exception
+      if @request.options[:raise_on] && @request.options[:raise_on].include?(code)
+        ::Kernel.raise ::HTTParty::ResponseError.new(@response), "Code #{code} - #{body}"
+      end
+    end
   end
 end
 
