@@ -27,7 +27,7 @@ RSpec.describe HTTParty::ConnectionAdapter do
     it "sets the options" do
       options = {foo: :bar}
       adapter = HTTParty::ConnectionAdapter.new(uri, options)
-      expect(adapter.options).to be options
+      expect(adapter.options.keys).to include(:verify, :verify_peer, :foo)
     end
   end
 
@@ -325,6 +325,19 @@ RSpec.describe HTTParty::ConnectionAdapter do
         end
       end
 
+      context 'when providing nil as proxy address' do
+        let(:uri) { URI 'http://noproxytest.com' }
+        let(:options) { {http_proxyaddr: nil} }
+
+        it { is_expected.not_to be_a_proxy }
+
+        it "does pass nil proxy parameters to the connection, this forces to not use a proxy" do
+          http = Net::HTTP.new("noproxytest.com")
+          expect(Net::HTTP).to receive(:new).once.with("noproxytest.com", 80, nil, nil, nil, nil).and_return(http)
+          adapter.connection
+        end
+      end
+
       context 'when not providing a proxy address' do
         let(:uri) { URI 'http://proxytest.com' }
 
@@ -372,6 +385,13 @@ RSpec.describe HTTParty::ConnectionAdapter do
             expect(subject.verify_mode).to eq(OpenSSL::SSL::VERIFY_PEER)
           end
 
+          context "when options include verify=false" do
+            let(:options) { {pem: pem, pem_password: "password", verify: false} }
+
+            it "should not verify the certificate" do
+              expect(subject.verify_mode).to eq(OpenSSL::SSL::VERIFY_NONE)
+            end
+          end
           context "when options include verify_peer=false" do
             let(:options) { {pem: pem, pem_password: "password", verify_peer: false} }
 
@@ -423,6 +443,13 @@ RSpec.describe HTTParty::ConnectionAdapter do
             expect(subject.verify_mode).to eq(OpenSSL::SSL::VERIFY_PEER)
           end
 
+          context "when options include verify=false" do
+            let(:options) { {p12: p12, p12_password: "password", verify: false} }
+
+            it "should not verify the certificate" do
+              expect(subject.verify_mode).to eq(OpenSSL::SSL::VERIFY_NONE)
+            end
+          end
           context "when options include verify_peer=false" do
             let(:options) { {p12: p12, p12_password: "password", verify_peer: false} }
 
