@@ -17,6 +17,34 @@ RSpec.describe Net::HTTPHeader::DigestAuthenticator do
     @digest.cookie_header
   end
 
+  context 'Net::HTTPHeader#digest_auth' do
+    let(:headers) {
+      (Class.new do 
+        include Net::HTTPHeader
+        def initialize
+          @header = {}
+        end
+      end).new
+    } 
+
+    let(:response){
+      (Class.new do 
+        include Net::HTTPHeader
+        def initialize
+          @header = {}
+          self['WWW-Authenticate'] = 
+          'Digest realm="testrealm@host.com", qop="auth,auth-int", nonce="dcd98b7102dd2f0e8b11d0f600bfb0c093", opaque="5ccc069c403ebaf9f0171e9517f40e41"'
+        end
+      end).new
+    }
+
+    it 'should set the authorization header' do 
+      expect(headers['authorization']).to be_nil
+      headers.digest_auth('user','pass', response)
+      expect(headers['authorization']).to_not be_empty
+    end 
+  end 
+
   context "with a cookie value in the response header" do
     before do
       @digest = setup_digest({
@@ -152,6 +180,16 @@ RSpec.describe Net::HTTPHeader::DigestAuthenticator do
     end
   end
 
+  context "with http basic auth response when net digest auth expected" do
+    it "should not fail" do
+      @digest = setup_digest({
+                               'www-authenticate' => 'WWW-Authenticate: Basic realm="testrealm.com""'
+                           })
+
+      expect(authorization_header).to include("Digest")
+    end
+  end
+
   context "with multiple authenticate headers" do
     before do
       @digest = setup_digest({
@@ -188,4 +226,43 @@ RSpec.describe Net::HTTPHeader::DigestAuthenticator do
       expect(authorization_header).to include(%(response="#{request_digest}"))
     end
   end
+  
+  context "with algorithm specified" do
+    before do
+      @digest = setup_digest({
+                              'www-authenticate' => 'Digest realm="myhost@testrealm.com", nonce="NONCE", qop="auth", algorithm=MD5'
+                             })
+    end
+    
+    it "should recognise algorithm was specified" do
+      expect( @digest.send :algorithm_present? ).to be(true)
+    end
+    
+    it "should set the algorithm header" do
+      expect(authorization_header).to include('algorithm="MD5"')
+    end
+  end
+
+  context "with md5-sess algorithm specified" do
+    before do
+      @digest = setup_digest({
+                              'www-authenticate' => 'Digest realm="myhost@testrealm.com", nonce="NONCE", qop="auth", algorithm=MD5-sess'
+                             })
+    end
+    
+    it "should recognise algorithm was specified" do
+      expect( @digest.send :algorithm_present? ).to be(true)
+    end
+    
+    it "should set the algorithm header" do
+      expect(authorization_header).to include('algorithm="MD5-sess"')
+    end
+    
+    it "should set response using md5-sess algorithm" do
+      request_digest = "md5(md5(md5(Mufasa:myhost@testrealm.com:Circle Of Life):NONCE:md5(deadbeef)):NONCE:00000001:md5(deadbeef):auth:md5(GET:/dir/index.html))"
+      expect(authorization_header).to include(%(response="#{request_digest}"))
+    end
+    
+  end
+  
 end
