@@ -129,6 +129,11 @@ RSpec.describe HTTParty do
         .and_return(double("mock response", perform: nil))
     end
 
+    it "does not modify default_options when no arguments are passed" do
+      @klass.headers
+      expect(@klass.default_options[:headers]).to eq(nil)
+    end
+
     it "should default to empty hash" do
       expect(@klass.headers).to eq({})
     end
@@ -171,11 +176,11 @@ RSpec.describe HTTParty do
         @klass.get('', cookies: {type: 'snickerdoodle'})
       end
 
-      it 'doesnt modify default_options' do
+      it 'doesnt modify default headers' do
         expect(@klass.headers).to eq({})
         expect_headers('cookie' => 'type=snickerdoodle')
         @klass.get('', cookies: {type: 'snickerdoodle'})
-        expect(@klass.default_options[:headers]).to eq({})
+        expect(@klass.headers).to eq({})
       end
 
       it 'adds optional cookies to the optional headers' do
@@ -328,7 +333,8 @@ RSpec.describe HTTParty do
 
     it "should be able parse response with custom parser" do
       @klass.parser parser
-      FakeWeb.register_uri(:get, 'http://twitter.com/statuses/public_timeline.xml', body: 'tweets')
+      stub_request(:get, 'http://twitter.com/statuses/public_timeline.xml')
+        .to_return(body: 'tweets')
       custom_parsed_response = @klass.get('http://twitter.com/statuses/public_timeline.xml')
       expect(custom_parsed_response[:sexy]).to eq(true)
     end
@@ -351,6 +357,45 @@ RSpec.describe HTTParty do
     end
   end
 
+  describe "uri_adapter" do
+
+    require 'forwardable'
+    class CustomURIAdaptor
+      extend Forwardable
+      def_delegators :@uri, :userinfo, :relative?, :query, :query=, :scheme, :path, :host, :port
+
+      def initialize uri
+        @uri = uri
+      end
+
+      def self.parse uri
+        new URI.parse uri
+      end
+    end
+
+    let(:uri_adapter) { CustomURIAdaptor }
+
+    it "should set the uri_adapter" do
+      @klass.uri_adapter uri_adapter
+      expect(@klass.default_options[:uri_adapter]).to be uri_adapter
+    end
+
+    it "should raise an ArgumentError if uri_adapter doesn't implement parse method" do
+      expect do
+        @klass.uri_adapter double()
+      end.to raise_error(ArgumentError)
+    end
+
+
+    it "should process a request with a uri instance parsed from the uri_adapter" do
+      uri = 'http://foo.com/bar'
+      stub_request(:get, uri).to_return(body: 'stuff')
+      @klass.uri_adapter uri_adapter
+      expect(@klass.get(uri).parsed_response).to eq('stuff')
+    end
+
+  end
+
   describe "connection_adapter" do
     let(:uri) { 'http://google.com/api.json' }
     let(:connection_adapter) { double('CustomConnectionAdapter') }
@@ -377,7 +422,7 @@ RSpec.describe HTTParty do
         expect(o[:connection_adapter_options]).to eq(connection_adapter_options)
         HTTParty::ConnectionAdapter.call(u, o)
       }.with(URI.parse(uri), kind_of(Hash))
-      FakeWeb.register_uri(:get, uri, body: 'stuff')
+      stub_request(:get, uri).to_return(body: 'stuff')
       @klass.connection_adapter connection_adapter, connection_adapter_options
       expect(@klass.get(uri).parsed_response).to eq('stuff')
     end
@@ -486,6 +531,22 @@ RSpec.describe HTTParty do
     end
   end
 
+  describe ".raise_on" do
+    context 'when parameters is an array' do
+      it 'sets raise_on option' do
+        @klass.raise_on [500, 404]
+        expect(@klass.default_options[:raise_on]).to contain_exactly(404, 500)
+      end
+    end
+
+    context 'when parameters is a fixnum' do
+      it 'sets raise_on option' do
+        @klass.raise_on 404
+        expect(@klass.default_options[:raise_on]).to contain_exactly(404)
+      end
+    end
+  end
+
   describe "with explicit override of automatic redirect handling" do
     before do
       @request = HTTParty::Request.new(Net::HTTP::Get, 'http://api.foo.com/v1', format: :xml, no_follow: true)
@@ -547,6 +608,39 @@ RSpec.describe HTTParty do
         @klass.options('/foo', no_follow: true)
       end.to raise_error(HTTParty::RedirectionTooDeep) {|e| expect(e.response.body).to eq('first redirect')}
     end
+
+    it "should fail with redirected MKCOL" do
+      expect do
+        @klass.mkcol('/foo', no_follow: true)
+      end.to raise_error(HTTParty::RedirectionTooDeep) {|e| expect(e.response.body).to eq('first redirect')}
+    end
+  end
+
+  describe "head requests should follow redirects requesting HEAD only" do
+    before do
+      allow(HTTParty::Request).to receive(:new).
+        and_return(double("mock response", perform: nil))
+    end
+
+    it "should remain HEAD request across redirects, unless specified otherwise" do
+      expect(@klass).to receive(:ensure_method_maintained_across_redirects).with({})
+      @klass.head('/foo')
+    end
+
+  end
+
+  describe "#ensure_method_maintained_across_redirects" do
+    it "should set maintain_method_across_redirects option if unspecified" do
+      options = {}
+      @klass.send(:ensure_method_maintained_across_redirects, options)
+      expect(options[:maintain_method_across_redirects]).to be_truthy
+    end
+
+    it "should not set maintain_method_across_redirects option if value is present" do
+      options = { maintain_method_across_redirects: false }
+      @klass.send(:ensure_method_maintained_across_redirects, options)
+      expect(options[:maintain_method_across_redirects]).to be_falsey
+    end
   end
 
   describe "with multiple class definitions" do
@@ -767,9 +861,11 @@ RSpec.describe HTTParty do
     end
 
     it "should accept webcal URIs" do
-      stub_http_response_with('google.html')
+      uri = 'http://google.com/'
+      stub_request(:get, uri).to_return(body: 'stuff')
+      uri = 'webcal://google.com/'
       expect do
-        HTTParty.get('webcal://google.com')
+        HTTParty.get(uri)
       end.not_to raise_error
     end
 
